@@ -5,6 +5,7 @@ to document how deprecations should eventually be turned into errors.
 """
 from __future__ import division, absolute_import, print_function
 
+import datetime
 import sys
 import operator
 import warnings
@@ -12,7 +13,116 @@ import warnings
 import numpy as np
 from numpy.testing import (
     run_module_suite, assert_raises, assert_warns, assert_no_warnings,
-    assert_array_equal, assert_)
+    assert_array_equal, assert_, dec)
+
+try:
+    import pytz
+    _has_pytz = True
+except ImportError:
+    _has_pytz = False
+
+
+class _VisibleDeprecationTestCase(object):
+    # Just as warning: warnings uses re.match, so the start of this message
+    # must match.
+    message = ''
+
+    def setUp(self):
+        self.warn_ctx = warnings.catch_warnings(record=True)
+        self.log = self.warn_ctx.__enter__()
+
+        # Do *not* ignore other DeprecationWarnings. Ignoring warnings
+        # can give very confusing results because of
+        # http://bugs.python.org/issue4180 and it is probably simplest to
+        # try to keep the tests cleanly giving only the right warning type.
+        # (While checking them set to "error" those are ignored anyway)
+        # We still have them show up, because otherwise they would be raised
+        warnings.filterwarnings("always", category=np.VisibleDeprecationWarning)
+        warnings.filterwarnings("always", message=self.message,
+                                category=np.VisibleDeprecationWarning)
+
+    def tearDown(self):
+        self.warn_ctx.__exit__()
+
+    def assert_deprecated(self, function, num=1, ignore_others=False,
+                          function_fails=False,
+                          exceptions=(np.VisibleDeprecationWarning,),
+                          args=(), kwargs={}):
+        """Test if VisibleDeprecationWarnings are given and raised.
+
+        This first checks if the function when called gives `num`
+        VisibleDeprecationWarnings, after that it tries to raise these
+        VisibleDeprecationWarnings and compares them with `exceptions`.
+        The exceptions can be different for cases where this code path
+        is simply not anticipated and the exception is replaced.
+
+        Parameters
+        ----------
+        f : callable
+            The function to test
+        num : int
+            Number of VisibleDeprecationWarnings to expect. This should
+            normally be 1.
+        ignore_other : bool
+            Whether warnings of the wrong type should be ignored (note that
+            the message is not checked)
+        function_fails : bool
+            If the function would normally fail, setting this will check for
+            warnings inside a try/except block.
+        exceptions : Exception or tuple of Exceptions
+            Exception to expect when turning the warnings into an error.
+            The default checks for DeprecationWarnings. If exceptions is
+            empty the function is expected to run successfull.
+        args : tuple
+            Arguments for `f`
+        kwargs : dict
+            Keyword arguments for `f`
+        """
+        # reset the log
+        self.log[:] = []
+
+        try:
+            function(*args, **kwargs)
+        except (Exception if function_fails else tuple()):
+            pass
+
+        # just in case, clear the registry
+        num_found = 0
+        for warning in self.log:
+            if warning.category is np.VisibleDeprecationWarning:
+                num_found += 1
+            elif not ignore_others:
+                raise AssertionError(
+                        "expected DeprecationWarning but got: %s" %
+                        (warning.category,))
+        if num is not None and num_found != num:
+            msg = "%i warnings found but %i expected." % (len(self.log), num)
+            lst = [w.category for w in self.log]
+            raise AssertionError("\n".join([msg] + lst))
+
+        with warnings.catch_warnings():
+            warnings.filterwarnings("error", message=self.message,
+                                    category=np.VisibleDeprecationWarning)
+            try:
+                function(*args, **kwargs)
+                if exceptions != tuple():
+                    raise AssertionError(
+                            "No error raised during function call")
+            except exceptions:
+                if exceptions == tuple():
+                    raise AssertionError(
+                            "Error raised during function call")
+
+    def assert_not_deprecated(self, function, args=(), kwargs={}):
+        """Test if VisibleDeprecationWarnings are given and raised.
+
+        This is just a shorthand for:
+
+        self.assert_deprecated(function, num=0, ignore_others=True,
+                        exceptions=tuple(), args=args, kwargs=kwargs)
+        """
+        self.assert_deprecated(function, num=0, ignore_others=True,
+                        exceptions=tuple(), args=args, kwargs=kwargs)
 
 
 class _DeprecationTestCase(object):
@@ -89,7 +199,7 @@ class _DeprecationTestCase(object):
         if num is not None and num_found != num:
             msg = "%i warnings found but %i expected." % (len(self.log), num)
             lst = [w.category for w in self.log]
-            raise AssertionError("\n".join([msg] + [lst]))
+            raise AssertionError("\n".join([msg] + lst))
 
         with warnings.catch_warnings():
             warnings.filterwarnings("error", message=self.message,
@@ -116,7 +226,7 @@ class _DeprecationTestCase(object):
                         exceptions=tuple(), args=args, kwargs=kwargs)
 
 
-class TestFloatNonIntegerArgumentDeprecation(_DeprecationTestCase):
+class TestFloatNonIntegerArgumentDeprecation(_VisibleDeprecationTestCase):
     """
     These test that ``DeprecationWarning`` is given when you try to use
     non-integers as arguments to for indexing and slicing e.g. ``a[0.0:5]``
@@ -255,7 +365,7 @@ class TestFloatNonIntegerArgumentDeprecation(_DeprecationTestCase):
         self.assert_deprecated(np.min, num=2, args=(d, (.2, 1.2)))
 
 
-class TestBooleanArgumentDeprecation(_DeprecationTestCase):
+class TestBooleanArgumentDeprecation(_VisibleDeprecationTestCase):
     """This tests that using a boolean as integer argument/indexing is
     deprecated.
 
@@ -279,7 +389,7 @@ class TestBooleanArgumentDeprecation(_DeprecationTestCase):
         self.assert_deprecated(lambda: a[False, 0, 0], exceptions=IndexError)
 
 
-class TestArrayToIndexDeprecation(_DeprecationTestCase):
+class TestArrayToIndexDeprecation(_VisibleDeprecationTestCase):
     """This tests that creating an an index from an array is deprecated
     if the array is not 0d.
 
@@ -301,7 +411,7 @@ class TestArrayToIndexDeprecation(_DeprecationTestCase):
         # Check slicing. Normal indexing checks arrays specifically.
         self.assert_deprecated(lambda: a[a:a:a], exceptions=(), num=3)
 
-class TestNonIntegerArrayLike(_DeprecationTestCase):
+class TestNonIntegerArrayLike(_VisibleDeprecationTestCase):
     """Tests that array likes, i.e. lists give a deprecation warning
     when they cannot be safely cast to an integer.
     """
@@ -331,7 +441,7 @@ class TestNonIntegerArrayLike(_DeprecationTestCase):
             #assert_raises(FutureWarning, a.__getitem__, True)
 
 
-class TestMultipleEllipsisDeprecation(_DeprecationTestCase):
+class TestMultipleEllipsisDeprecation(_VisibleDeprecationTestCase):
     message = "an index can only have a single Ellipsis \(`...`\); replace " \
               "all but one with slices \(`:`\)."
 
@@ -340,7 +450,7 @@ class TestMultipleEllipsisDeprecation(_DeprecationTestCase):
         self.assert_deprecated(a.__getitem__, args=((Ellipsis, Ellipsis),))
 
         with warnings.catch_warnings():
-            warnings.filterwarnings('ignore', '', DeprecationWarning)
+            warnings.filterwarnings('ignore', '', np.VisibleDeprecationWarning)
             # Just check that this works:
             b = a[...,...]
             assert_array_equal(a, b)
@@ -394,8 +504,8 @@ class TestRankDeprecation(_DeprecationTestCase):
 
 
 class TestComparisonDeprecations(_DeprecationTestCase):
-    """This tests the deprecation, for non-elementwise comparison logic.
-    This used to mean that when an error occured during element-wise comparison
+    """This tests the deprecation, for non-element-wise comparison logic.
+    This used to mean that when an error occurred during element-wise comparison
     (i.e. broadcasting) NotImplemented was returned, but also in the comparison
     itself, False was given instead of the error.
 
@@ -423,13 +533,13 @@ class TestComparisonDeprecations(_DeprecationTestCase):
         b = np.array(['a', 'b', 'c'])
         assert_raises(ValueError, lambda x, y: x == y, a, b)
 
-        # The empty list is not cast to string, this is only to document
+        # The empty list is not cast to string, as this is only to document
         # that fact (it likely should be changed). This means that the
         # following works (and returns False) due to dtype mismatch:
         a == []
 
     def test_none_comparison(self):
-        # Test comparison of None, which should result in elementwise
+        # Test comparison of None, which should result in element-wise
         # comparison in the future. [1, 2] == None should be [False, False].
         with warnings.catch_warnings():
             warnings.filterwarnings('always', '', FutureWarning)
@@ -442,7 +552,7 @@ class TestComparisonDeprecations(_DeprecationTestCase):
             assert_raises(FutureWarning, operator.ne, np.arange(3), None)
 
     def test_scalar_none_comparison(self):
-        # Scalars should still just return false and not give a warnings.
+        # Scalars should still just return False and not give a warnings.
         # The comparisons are flagged by pep8, ignore that.
         with warnings.catch_warnings(record=True) as w:
             warnings.filterwarnings('always', '', FutureWarning)
@@ -457,9 +567,9 @@ class TestComparisonDeprecations(_DeprecationTestCase):
             assert_(np.datetime64('NaT') != None)
         assert_(len(w) == 0)
 
-        # For documentaiton purpose, this is why the datetime is dubious.
+        # For documentation purposes, this is why the datetime is dubious.
         # At the time of deprecation this was no behaviour change, but
-        # it has to be considered when the deprecations is done.
+        # it has to be considered when the deprecations are done.
         assert_(np.equal(np.datetime64('NaT'), None))
 
     def test_void_dtype_equality_failures(self):
@@ -508,7 +618,7 @@ class TestComparisonDeprecations(_DeprecationTestCase):
                         with warnings.catch_warnings() as l:
                             warnings.filterwarnings("always")
                             assert_raises(TypeError, f, arg1, arg2)
-                            assert not l
+                            assert_(not l)
                     else:
                         # py2
                         assert_warns(DeprecationWarning, f, arg1, arg2)
@@ -569,8 +679,8 @@ class TestIdentityComparisonDeprecations(_DeprecationTestCase):
 class TestAlterdotRestoredotDeprecations(_DeprecationTestCase):
     """The alterdot/restoredot functions are deprecated.
 
-    These functions no longer do anything in numpy 1.10, so should not be
-    used.
+    These functions no longer do anything in numpy 1.10, so
+    they should not be used.
 
     """
 
@@ -581,7 +691,7 @@ class TestAlterdotRestoredotDeprecations(_DeprecationTestCase):
 
 class TestBooleanIndexShapeMismatchDeprecation():
     """Tests deprecation for boolean indexing where the boolean array
-    does not match the input array along the given diemsions.
+    does not match the input array along the given dimensions.
     """
     message = r"boolean index did not match indexed array"
 
@@ -617,6 +727,27 @@ class TestFullDefaultDtype(object):
         assert_no_warnings(np.full, 1, 1, float)
 
 
+class TestDatetime64Timezone(_DeprecationTestCase):
+    """Parsing of datetime64 with timezones deprecated in 1.11.0, because
+    datetime64 is now timezone naive rather than UTC only.
+
+    It will be quite a while before we can remove this, because, at the very
+    least, a lot of existing code uses the 'Z' modifier to avoid conversion
+    from local time to UTC, even if otherwise it handles time in a timezone
+    naive fashion.
+    """
+    def test_string(self):
+        self.assert_deprecated(np.datetime64, args=('2000-01-01T00+01',))
+        self.assert_deprecated(np.datetime64, args=('2000-01-01T00Z',))
+
+    @dec.skipif(not _has_pytz, "The pytz module is not available.")
+    @dec.knownfailureif(sys.version_info[0:2] < (2, 7))
+    def test_datetime(self):
+        tz = pytz.timezone('US/Eastern')
+        dt = datetime.datetime(2000, 1, 1, 0, 0, tzinfo=tz)
+        self.assert_deprecated(np.datetime64, args=(dt,))
+
+
 class TestNonCContiguousViewDeprecation(_DeprecationTestCase):
     """View of non-C-contiguous arrays deprecated in 1.11.0.
 
@@ -631,5 +762,48 @@ class TestNonCContiguousViewDeprecation(_DeprecationTestCase):
         self.assert_deprecated(np.ones((2,2)).T.view, args=(np.int8,))
 
 
+class TestInvalidOrderParameterInputForFlattenArrayDeprecation(_DeprecationTestCase):
+    """Invalid arguments to the ORDER parameter in array.flatten() should not be
+    allowed and should raise an error.  However, in the interests of not breaking
+    code that may inadvertently pass invalid arguments to this parameter, a
+    DeprecationWarning will be issued instead for the time being to give developers
+    time to refactor relevant code.
+    """
+
+    def test_flatten_array_non_string_arg(self):
+        x = np.zeros((3, 5))
+        self.message = ("Non-string object detected for "
+                        "the array ordering. Please pass "
+                        "in 'C', 'F', 'A', or 'K' instead")
+        self.assert_deprecated(x.flatten, args=(np.pi,))
+
+    def test_flatten_array_invalid_string_arg(self):
+        # Tests that a DeprecationWarning is raised
+        # when a string of length greater than one
+        # starting with "C", "F", "A", or "K" (case-
+        # and unicode-insensitive) is passed in for
+        # the ORDER parameter. Otherwise, a TypeError
+        # will be raised!
+
+        x = np.zeros((3, 5))
+        self.message = ("Non length-one string passed "
+                        "in for the array ordering. Please "
+                        "pass in 'C', 'F', 'A', or 'K' instead")
+        self.assert_deprecated(x.flatten, args=("FACK",))
+
+
+class TestTestDeprecated(object):
+    def test_assert_deprecated(self):
+        test_case_instance = _DeprecationTestCase()
+        test_case_instance.setUp()
+        assert_raises(AssertionError,
+                      test_case_instance.assert_deprecated,
+                      lambda: None)
+
+        def foo():
+            warnings.warn("foo", category=DeprecationWarning)
+
+        test_case_instance.assert_deprecated(foo)
+
 if __name__ == "__main__":
     run_module_suite()
